#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#

#  Copyright (C) 2014-2015  Rafael Senties Martinelli <rsm@imap.cc>
#
#  This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License 3 as published by
#   the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
#

""" Special thanks to the VideoLAN Team.  """


"""

 To Do:
 
	#130 : When a button is on focus and the enter key is pressed there is a bug.
	- Display the mouse pointer when the toolbox is shown
	
	#100 : Replace the string hack with Gtk code

	Add more vlc options?
	
"""



from gi.repository import Gtk, GObject, GLib, Gdk, GdkX11
import sys, threading, time, os, datetime

from gettext import gettext as _


# local imports
from CustomGtk import GTK_file_chooser
import vlc


# Create a single vlc.Instance() to be shared by (possible) multiple players.
instance = vlc.Instance('--mouse-hide-timeout=5')


def get_active_window_title():
	output= os.popen('''xprop -id $(xprop -root _NET_ACTIVE_WINDOW | cut -d ' ' -f 5) WM_NAME''')
	output=str(output.read())
	
	try:
		return output.split('''= "''')[1][:-2]
	except:
		return None



def vlc_play_video(media_player,fname,root_window,thread, position):
	
	if os.path.exists(fname):
	
		media=instance.media_new(fname)
		media.parse()
	
		media_player.set_media(media)
		media_player.play()
		
		if position > 0:
			media_player.set_position(position)
		
		
		if not thread:
			root_window.set_title(media.get_meta(0))	
			root_window.show_all()
		else:
			Gdk.threads_enter()
			root_window.set_title(media.get_meta(0))
			Gdk.threads_leave()
			
			Gdk.threads_enter()
			root_window.set_title(media.get_meta(0))
			Gdk.threads_leave()


class VLCWidget(Gtk.DrawingArea):
	"""Simple VLC widget.

	Its player can be controlled through the 'player' attribute, which
	is a vlc.MediaPlayer() instance.
	"""
	def __init__(self, root_window, *p):
		Gtk.DrawingArea.__init__(self)
		self.drawing_area=self.get_property('window')
		
		self.player=instance.media_player_new()
		self.window_root=root_window
		
		
		self.the_player_is_at_the_top_of_the_desktop=None
		self.root_menu=None # this is a dirty hack for make the menu work under Ubuntu
		
		
		def handle_embed(*args):
			"""This code makes that the player does not starts in a new window"""
			self.player.set_xwindow(self.get_window().get_xid())
			return True

		self.connect("map", handle_embed)
		
		self.set_size_request(320, 200)
		
		self.add_events(Gdk.EventMask.BUTTON_PRESS_MASK)
		self.connect('button-press-event', self.on_mouse_button_press)
		
		self.add_events(Gdk.EventMask.POINTER_MOTION_MASK)
		self.connect('motion_notify_event', self.on_mouse_motion)
		
		
		self.add_events(Gdk.EventMask.SCROLL_MASK)
		self.connect('scroll_event', self.on_mouse_scroll)
		
		self.mouse_time=time.time()
		self.increment=5 # % increment the volume
		
		
	def quit(self, widget):
		self.window_root.hide()
		self.player.stop()
		
	def on_mouse_button_press(self, widget, event):
		
		type=event.type
		button=event.button
		player_state=self.player.get_state()
		
		#if type==Gdk.BUTTON_PRESS:
		if "GDK_BUTTON_PRESS" in str(type):							################################### Hack #100
		
			if button==1:
				if "Playing" in str(player_state) and self.the_player_is_at_the_top_of_the_desktop:
					self.player.pause()
				else:
					self.player.play()
			elif button==3:
				"""
					Audio, Sound and Subtitles Menu
				"""

				self.root_menu=None
				self.root_menu=Gtk.Menu()
				# The tree menus are dynamically created and its childrens too
				
				
				menus_list=[Gtk.ImageMenuItem("Audio"),
							Gtk.ImageMenuItem("Video"),
							Gtk.ImageMenuItem("Subtitles"),
							]
							
							
				menus_commands=[self.player.audio_get_track_count(),
								self.player.audio_get_track(),
								self.player.video_get_track_count(),
								self.player.video_get_track(),
								self.player.video_get_spu_count(),
								self.player.video_get_spu(),
								]
								


				for k in range(0, len(menus_list)):
				
					count=menus_commands[k*2]
					current=menus_commands[k*2+1]

			
					submenu=Gtk.Menu()
					menus_list[k].set_submenu(submenu)
					
					for i in range(0,count):
						
						
						if i == 0:
							track=Gtk.CheckMenuItem("None")
						else:
							track=Gtk.CheckMenuItem("Track "+str(i))
							
						
						if i == current:
							track.set_active(True)
						else:
							track.set_active(False)
							
							
						def callback(widget,k,i):
							if k==0:
								state=self.player.audio_set_track(i)
							if k==1:
								state=self.player.video_set_track(i)
							if k==2:
								state=self.player.video_set_spu(i)

							return True
							

						track.connect("activate", callback, k, i)
						submenu.append(track)
					
					if k==2: # subtitles menu
						subs=Gtk.ImageMenuItem("From File")
						subs.connect("activate", self.set_subtitles_from_file)
						submenu.append(subs)
					

					self.root_menu.append(menus_list[k])

				
				
				menu=Gtk.ImageMenuItem("Close")
				menu.connect('activate',self.quit)
				self.root_menu.append(menu)
				
				
				
				
				self.root_menu.show_all()
				
				
				

				self.root_menu.popup(None, None, lambda menu, data: (event.get_root_coords()[0],
																  event.get_root_coords()[1], True),
																	None, event.button, event.time)
				
				
				return True

				
				
				
				
				
				
		
		
		#elif type==Gdk._2BUTTON_PRESS:	
		elif "GDK_2BUTTON_PRESS" in str(type):					 	################################## Hack #100	
			if button==1:
				state=self.window_root.get_window().get_state()

				#if state==Gdk.WindowState.FULLSCREEN:
				if "GDK_WINDOW_STATE_FULLSCREEN" in str(state):  	 ################################### Hack #100
					self.window_root.unfullscreen()
				else:
					self.window_root.fullscreen()
		
		
	def set_subtitles_from_file(self, widget):

		path=GTK_file_chooser(self.window_root,None)
		if path:
			result=self.player.video_set_subtitle_file(path)
					
			print result
			
		return True
		
		
	def on_mouse_motion(self, widget, event):
		if self.window_root.is_active():
			self.mouse_time=int(time.time())
		
		
	def on_mouse_scroll(self,widget, event):
		
		direction=event.direction
	
		if "SCROLL_UP" in str(direction):                              ################################### Hack #100
			self.volume_up()
			
		elif "SCROLL_DOWN" in str(direction):							################################### Hack #100
			self.volume_down()
				
				
	def volume_up(self):
		actual_volume=self.player.audio_get_volume()
		if actual_volume+self.increment <= 200:
			self.player.audio_set_volume(actual_volume+self.increment)
		else:
			self.player.audio_set_volume(200)
		
	def volume_down(self):
		actual_volume=self.player.audio_get_volume()
		if actual_volume >= self.increment:
			self.player.audio_set_volume(actual_volume-self.increment)
		else:
			self.player.audio_set_volume(0)	


class DecoratedVLCWidget(Gtk.VBox):
	"""Decorated VLC widget.

	VLC widget decorated with a player control toolbox.

	Its player can be controlled through the 'player' attribute, which
	is a Player instance.
	"""
	def __init__(self, root_window, *p):
		
		Gtk.VBox.__init__(self)
		self._vlc_widget = VLCWidget(root_window, *p)
		self.player = self._vlc_widget.player
		self.pack_start(self._vlc_widget, True, True, 0)
		self._toolbox = self.control_toolbox()
		
		self._vlc_widget.modify_bg(Gtk.StateFlags.NORMAL, Gdk.color_parse('#000000'))
		
		
		self.pack_start(self._toolbox, False, True, 0)

		self.window_root=root_window
		self.window_root.connect('key-press-event', self.on_key_pressed)
		self.window_root.connect('delete-event', self.on_button_destroy_window)

		"""Scan the mouse activity"""
		thrd=threading.Thread(target=self.on_mouse_motion)
		thrd.start()
				
		"""Update the toolbox"""
		thrd=threading.Thread(target=self.scan_the_player_activity)
		thrd.start()
		
		
		self.stopped_position=0 
		

	def control_toolbox(self):
		"""Return a player control toolbox"""

		self.toolbox=Gtk.Box()
		self.toolbox.set_hexpand(True)
		
		self.button_play_pause=Gtk.ToolButton(Gtk.STOCK_MEDIA_PLAY)
		self.button_play_pause.connect("clicked", self.on_button_play_pause_clicked)
		self.button_play_pause.set_can_focus(False)
		
		self.button_stop=Gtk.ToolButton(Gtk.STOCK_MEDIA_STOP)
		self.button_stop.connect("clicked", self.on_button_player_stop)
		self.button_stop.set_can_focus(False)
		
		self.button_restart=Gtk.ToolButton(Gtk.STOCK_MEDIA_PREVIOUS)
		self.button_restart.connect("clicked", self.on_button_restart_the_video)
		self.button_restart.set_can_focus(False)
		
		self.button_end_video=Gtk.ToolButton(Gtk.STOCK_MEDIA_NEXT)
		self.button_end_video.connect("clicked", self.on_button_end_the_video)
		self.button_end_video.set_can_focus(False)
		
		
		label_volume=Gtk.Label(" Volume: ")
		self.scale_volume=Gtk.Scale()
		self.scale_volume.set_range(0,200)
		self.scale_volume.set_size_request(100,-1)
		self.scale_volume.set_value_pos(Gtk.PositionType.LEFT)
		self.scale_volume.set_digits(0)
		self.scale_volume.connect('value_changed', self.scale_volume_changed)
		self.scale_volume.connect('button-press-event', self.scale_volume_click)
		self.scale_volume.connect('button-release-event', self.scale_volume_click)
		self.scale_volume.set_can_focus(False)
		self.scale_volume.set_margin_right(10)

		self.label_progress=Gtk.Label("00:00:00")
		self.label_progress.set_margin_right(5)
		
		self.scale_progress=Gtk.Scale()
		self.scale_progress.set_range(0,1)
		self.scale_progress.set_size_request(100,-1)
		self.scale_progress.set_draw_value(False)
		self.scale_progress.set_hexpand(True)
		self.scale_progress.set_can_focus(False)
		self.scale_progress.connect('value_changed', self.scale_progres_changed)
		self.scale_progress.connect('button-press-event', self.scale_button_press)
		self.scale_progress.connect('button-release-event', self.scale_button_release)
		self.scale_progress.set_margin_right(5)
		self.update_progress_bar=True

		self.label_lenght=Gtk.Label("00:00:00   ") 
		""" the spaces work better than the margin property
			because the margin property is creating a border on the window.
		"""
		#self.set_margin_right(10)
	
		self.toolbox.add(self.button_restart)
		self.toolbox.add(self.button_play_pause)
		self.toolbox.add(self.button_stop)
		self.toolbox.add(self.button_end_video)
		
		self.toolbox.add(label_volume)
		self.toolbox.add(self.scale_volume)
		
		self.toolbox.add(self.label_progress)
		self.toolbox.add(self.scale_progress)
		self.toolbox.add(self.label_lenght)
		
		
		self.toolbox.show_all()
		
		self.stoped_player_position=0
		
		return self.toolbox
		
	def on_button_destroy_window(self, data=None, data2=None):
		self.player.stop()
		self.stoped_player_position=0
		self.window_root.hide()
		
		return True # This prevents the window from being destroyed. This is important for vlist-player!
	
	
	def on_button_player_stop(self, data=None):
		self.stopped_position=self.player.get_position()
		self.player.stop()
		self.window_root.hide()
	
	def on_button_play_pause_clicked(self, button, data=None):
		"""This method modify the button "play/stop"""

		if not "Playing" in str(self.player.get_state()):
			self.button_play_pause.set_stock_id("gtk-media-pause")
			self.player.play()
		else:
			self.button_play_pause.set_stock_id("gtk-media-play")
			self.player.pause()
		

	def on_mouse_motion(self):
		"""This method hides the tool bar when there is no mouse activity"""
		
		while True:
			if int(time.time())-self._vlc_widget.mouse_time >= 4: #and "Playing" in str(self.player.get_state()):
				Gdk.threads_enter()
				self.toolbox.hide()
				Gdk.threads_leave()
			else:
				Gdk.threads_enter()
				self.toolbox.show()
				Gdk.threads_leave()
	
			time.sleep(0.5)
			
			
	def scan_the_player_activity(self):
		"""This method scans the state of the player to update the tool buttons, volume, play-stop etc"""
		
		while True:
			# Update the play-pause button
			if "Playing" in str(self.player.get_state()) and self.button_play_pause.get_stock_id()=="gtk-media-play":
				Gdk.threads_enter()
				self.button_play_pause.set_stock_id("gtk-media-pause")
				Gdk.threads_leave()
			elif not "Playing" in str(self.player.get_state()) and self.button_play_pause.get_stock_id()=="gtk-media-pause":
				Gdk.threads_enter()
				self.button_play_pause.set_stock_id("gtk-media-play")
				Gdk.threads_leave()
			
			"""
				Update the volume scale
			"""
			vlc_volume=self.player.audio_get_volume()	
			scale_value=self.scale_volume.get_value()
			
			if vlc_volume != scale_value:
				Gdk.threads_enter()
				self.scale_volume.set_value(vlc_volume)
				Gdk.threads_leave()
			
			"""
				Update the progress scale
			"""
			vlc_position=self.player.get_position()
			scale_value=self.scale_progress.get_value()
			
			if scale_value != vlc_position and self.update_progress_bar:
				Gdk.threads_enter()
				self.scale_progress.set_value(vlc_position)
				Gdk.threads_leave()
				
			if get_active_window_title()==self.window_root.get_title():
				self._vlc_widget.the_player_is_at_the_top_of_the_desktop=True
			else:
				self._vlc_widget.the_player_is_at_the_top_of_the_desktop=False

			"""
				Update the time of the player
			"""
			if 'Playing' in str(self.player.get_state()):
				video_lenght=datetime.timedelta(milliseconds=self.player.get_length()) 
				video_lenght=str(video_lenght).split('.')[0]+"   "
				Gdk.threads_enter()			
				self.label_lenght.set_text(video_lenght)
				Gdk.threads_leave()
				
				
				video_time=datetime.timedelta(milliseconds=self.player.get_time()) 
				video_time=str(video_time).split('.')[0]
				Gdk.threads_enter()	
				self.label_progress.set_text(video_time)
				Gdk.threads_leave()


			time.sleep(0.2)
		
	def on_key_pressed(self, widget, ev, data=None):
		"""This method connects the hotkeys of the root window"""


		esc_key=65307
		f11_key=65480
		space_bar=32
		enter_key=65293
		
		arrow_up=65362								
		arrow_down=65364
		
		#arrow_left=																	############ start																
		#arrow_next=																	############ back
		
		
		key=ev.keyval
		
		#print key
		
		if key==esc_key:
			self.window_root.unfullscreen()
		elif key==f11_key:
			self.window_root.fullscreen()
		elif key==space_bar or key==enter_key:
			
			#focus=self.window_root.get_focus()		########################## #130 . When a button has focus and the enter key is pressed.
			#if focus != self.button_play_pause:
			self.on_button_play_pause_clicked(None, None)
				
		elif key==arrow_up:
			self._vlc_widget.volume_up()
		elif key==arrow_down:
			self._vlc_widget.volume_down()
			

	def scale_volume_changed(self, data=None):
		self.player.audio_set_volume(int(self.scale_volume.get_value()))

	def scale_volume_click(self, scale, event):
		
		if event.button == 1:
			event.button = 2
		return False


	def scale_progres_changed(self, data=None):
		if not self.update_progress_bar:
			self.player.set_position(self.scale_progress.get_value())

	def scale_button_press(self, scale, event):
		self.update_progress_bar=False
		
		if event.button == 1:
			event.button = 2
		return False
		
		
	def scale_button_release(self, scale, event):
		self.update_progress_bar=True
		
		if event.button == 1:
			event.button = 2
		return False

	def on_button_restart_the_video(self, data=None, data2=None):
		self.player.set_position(0)
		
	def on_button_end_the_video(self, data=None, data2=None):
		self.player.set_position(1)


class IndependentVideoPlayer:
	"""This class allows to test the video player with out calling any external file (except a video to play)"""
	
	def __init__(self):
		
		root_window=Gtk.Window()
		
		self.vlc = DecoratedVLCWidget(root_window)
		
		fname='/home/rsm/programs/vlist-player/vlist-player/images/'
		
		media=instance.media_new(fname)
		media.parse()
		root_window.set_title(media.get_meta(0))
	
		self.vlc.player.set_media(media)
		
		
		root_window.add(self.vlc)
		root_window.show_all()
	

if __name__ == '__main__':
	"""This calls the video player for testing it"""


	GObject.threads_init()
	Gdk.threads_init()
        
	main=IndependentVideoPlayer()
	Gtk.main()
