#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#

#  Copyright (C) 2014-2015  Rafael Senties Martinelli <rsm@imap.cc>
#
#  This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License 3 as published by
#   the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software Foundation,
#   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.


from gi.repository import Gtk
from gi.repository.GdkPixbuf import Pixbuf, InterpType
import os, csv, shutil, random

# local imports
from CustomOs import *; CustomOs=CustomOs()
from Paths import *


class Serie(object):
	
	def __init__(self,path,data_path,recursive,random,keep_playing):
		
		self.path=path
		self.name=os.path.basename(path)
		
		if recursive=="true" or recursive == "True" or recursive == True:
			self.recursive=True
		else:
			self.recursive=False
			
		if random=="true" or random == "True" or random == True:
			self.random=True
		else:
			self.random=False
		
		if keep_playing=="true" or keep_playing == "True" or keep_playing == True:
			self.keep_playing=True
		else:
			self.keep_playing=False
			
			
		self.videos=[]
		self.nb_videos=0
		
		
		# Image Variables
		self.series_image_small_size=30
		self.series_image_big_size=160
		self.load_image()
			
			
		# change the name of the serie in case it has been renamed.
		if data_path and os.path.exists(data_path):
			file_name=os.path.basename(data_path)

			if file_name[:-4] != self.name:
				self.name=file_name[:-4]
		
		
		if os.path.exists(SERIE_PATH.format(self.name)):
	
			"""
				Get the number of rows of the file, this is important in case there be an error with the id's
			"""
			number_of_rows=0
			if os.path.exists(SERIE_PATH.format(self.name)):
				with open(SERIE_PATH.format(self.name), 'r') as f:
					list=csv.reader(f,delimiter='|')
					for row in list:
						number_of_rows+=1
	

			"""
				Load the videos (and their data) from the program files
			"""
			with open(SERIE_PATH.format(self.name), 'r') as f:
				list=csv.reader(f,delimiter='|')
				next(f)
				for row in list:
					try:
						path=row[1]
					except:
						print "error getting the path"
						path=False
					
					
					"""
						check for duplicates
					"""
					if path:
						add=True
						for video in self.videos:
							if video.get_path()==path:
								add=False
								break 
					else:
						add=False
					
					
					if add:
						try:
							id=int(row[0])
						except:
							print "error getting the id"
							id=False
							
						if id:
							
							"""
								check if the Id has already been used
							"""
							for video in self.videos:
								if id==video.get_id():
									"""Send the video to the end"""
									number_of_rows+=1
									id=number_of_rows
									print "Incrementing duplicated id"
							

							try:
								play=row[2]
							except:
								play='true'
							
							try:
								o_played=row[3]
							except:
								o_played='false'
							
							try:
								r_played=row[4]
							except:
								r_played='false'
							
							try:
								position=float(row[5])
							except:
								position=0.0
							
							try:
								display=row[6]
							except:
								display='true'
							
							video=Video(path,id)
							video.load_info(play,o_played,r_played,position,display)	

							
							self.videos.append(video)
							self.nb_videos+=1
	

		
		"""
			Get the videos from the folder. This will find new videos.
		"""
		
		for video_path in CustomOs.generate_list_from_videos_folder(self.path,self.recursive):
			add=True
				
			for video in self.videos:
				if video.get_path().strip()==video_path:
					add=False
					break
					
			if add:
				self.nb_videos+=1
				new_video=Video(video_path,self.nb_videos)
				
				
				if os.path.exists(new_video.get_path()):
					new_video.set_state_new()
				else:
					# In case it is a broken link:
					new_video.update_state()
				
				self.videos.append(new_video)

						
		self.clean_episodes()
		self.update_ids()	# this is in case there was videos with duplicated ids
		self.write_data()

	def find_serie(self, path):
		self.path=path
		self.load_image()
		self.find_videos(path)
		self.update_not_hiden_videos()
		self.write_data()


	def load_image(self):
		"""
			Set the default image
		"""
		possible_image=self.path+"/.folder".replace("//","/")
		if os.path.exists(possible_image):
			image_path=possible_image
		elif os.path.exists(possible_image+".png"):
			image_path=possible_image+".png"
		elif os.path.exists(possible_image+".jpg"):
			image_path=possible_image+".jpg"
		elif os.path.exists(possible_image+".jpeg"):
			image_path=possible_image+".jpeg"
		else:
			if os.path.exists(self.path):
				image_path=ICON_LOGO_MEDIUM
			else:
				image_path=ICON_ERROR_BIG
			
			
		self.pixbuf_small_video=Pixbuf.new_from_file_at_size(image_path,self.series_image_small_size,self.series_image_small_size)
		self.image_big=Gtk.Image.new_from_pixbuf(Pixbuf.new_from_file_at_size(image_path,self.series_image_big_size+60,self.series_image_big_size))

	def find_video(self, episode_name, new_path):
		
		"""
			change the path of the selected episode
		"""
		for episode in self.videos:
			if episode_name==episode.get_name():
				episode.set_path(new_path)
				break
				
		"""		
			Try to find videos other videos in case
			the name of a extension changed, or the start part
			of the episode.
			
			Ex 1:
			
				foo.ogg  -> foo
				faa.ogg	 -> faa
			
					or
					
				videos-video700	-> video700
				videos-video800	-> video800

																	Could this be more powerful and search other kind of sequence of strings?
			
			ex:
			
				video-VIDEO700 -> video-video700
				video-VIDEO800 -> video-video800
			
		"""
			
			
		# Find the pattern
		
		new_name=os.path.basename(new_path).strip()
		old_name=episode_name.strip()
		
		len_new_name=len(new_name)
		len_old_name=len(episode_name)
		
		found_videos=0
		if new_name in old_name or old_name in new_name:

			if len_new_name > len_old_name:
				patt=new_name.replace(old_name,'')
				
				# Check if there are other missing videos with the same pattern
				for video in self.videos:
					if not os.path.exists(video.get_path()):
						video_name=video.get_name()
						video_path=video.get_path()
						basedir=os.path.dirname(video_path)+"/"
						basedir.replace("//","/")
						
						if os.path.exists(basedir+patt+video_name):
							video.set_path(basedir+patt+video_name)
							found_videos+=1
							
						elif os.path.exists(video_path+patt):
							video.set_path(video_path+patt)
							found_videos+=1
			
			
			elif len_old_name > len_new_name:
				patt=old_name.replace(new_name,'')
				
				# Check if there are other missing videos with the same pattern
				for video in self.videos:
					if not os.path.exists(video.get_path()):
						
						possible_path=video.get_path().replace(patt,'')
						
						if os.path.exists(possible_path):
							video.set_path(possible_path)
							found_videos+=1
		
			
			self.write_data()
			
			if found_videos > 0:
				return found_videos
			else:
				return False
				
	def update_not_hiden_videos(self):
		i=0
		for video in self.videos:
			if video.get_display():
				i+=1
				
		self.nb_videos=i
				
	def find_videos(self, path):
		
		path=path+'/'
		path.replace('//','/')
		
		video_counter=0
		for video in self.videos:
			if not os.path.exists(video.get_path()):
				if os.path.exists(path+video.get_name()):
					video_counter+=1
					video.set_path(path+video.get_name())
				
				
		if video_counter > 0:
			self.write_data()
			
		return video_counter

			
	def ignore_video(self,del_video):
		""" Ignore a video from the serie by givin its name.
		"""
	
		for video in self.videos:
			if video.get_name()==del_video:
				
				if os.path.exists(video.get_path()):
					video.set_display(False)
				else:
					self.videos.remove(video)
		
				self.nb_videos-=1
				self.update_ids()
				self.write_data()
				
				self.update_not_hiden_videos()
				
				break
				
	def dont_ignore_video(self,video_name):
		for video in self.videos:
			if video.get_name()==video_name:

				video.set_display(True)

				self.nb_videos-=1
				self.update_ids()
				self.write_data()
				
				self.update_not_hiden_videos()
				
				break	
			

	def missing_videos(self,episodes_names):
		"""Return if from the selected videos there is some one missing"""
		for episode_name in episodes_names:
			for episode in self.videos:
				if episode.get_name()==episode_name:
					if not os.path.exists(episode.get_path()):
						return True
						
						
		return False
		

	def clean_episodes(self):
		
		list=[]
		for video in self.videos:
			if video and not os.path.exists(video.get_path()) and not video.get_display():
				'''Delete the hiden and unexisting videos'''
			else:
				list.append(video)
				
		self.videos=list
				


	def get_path_from_video_name(self,name):
		for video in self.videos:
			if video and video.get_name()==name:
				return video.get_path()
		
		return None

	def get_keep_playing(self):
		return self.keep_playing

	def set_keep_playing(self,bool):
		self.keep_playing=bool
		self.write_data()
		
	def get_random(self):
		return self.random
		
	def set_random(self,bool):
		self.random=bool
		self.write_data()

	def move_episodes(self,rows,up_down):
		""" Choices are "up" or "down" """
		
		if up_down=='down':
			rows.reverse()
		
		
		for row in rows:
			row=int(str(row))
				
			if row > 0:
				# move the video to its new position
				video_to_move=self.videos[row]
				self.videos.remove(video_to_move)
				
				if up_down=="down":
					self.videos.insert(row+1,video_to_move)
				elif up_down=="up":
					self.videos.insert(row-1,video_to_move)
				else:
					print "wront argument moving episode of the serie:"+self.name
					
		# set the new ids
		i=1
		for video in self.videos:
			video.set_id(i)
			i+=1
		
		self.write_data()


	def change_checkbox_state(self,episode_names,column,state):
		
		if isinstance(episode_names,str):
			episode_names=[episode_names]
		
		for episode_name in episode_names:
			for video in self.videos:
				
				if video.get_name()==episode_name:

					if column==3:
						video.set_play(state)
					elif column==4:
						video.set_o_played(state)
					elif column==5:
						video.set_r_played(state)

					break
		
					
		self.write_data()
		
		
	def mark_episode(self,episode,random,new_state):
		for video in self.videos:
			if video==episode:
				if random: 
					video.set_r_played(new_state)  	
				else:
					video.set_o_played(new_state)
		
				self.write_data()
				break
				
		
	
	def reset_data(self):
		for video in self.videos:
			video.set_play(True)
			video.set_o_played(False)
			video.set_r_played(False)
			video.set_position(0)
			
		self.write_data()
		
		
	def set_video_position(self,video_to_find,position):
		for video in self.videos:
			if video_to_find==video:
				video.set_position(position)
				
				self.write_data()
				
				break
						
	def update_ids(self):
		i=1
		for video in self.videos:
			video.set_id(i)
			i+=1
		
	
	def set_videos(self,list):
		self.videos=list
	
	def set_name(self,new_name):
		self.name=new_name
		
	def rename(self,new_name):
		# update the data file
		if os.path.exists(SERIE_PATH.format(self.name)):
			os.rename(SERIE_PATH.format(self.name),SERIE_PATH.format(new_name))
		
		# update the class
		self.name=new_name
		
	def set_image(self,path):
		if os.path.exists(path):
			self.pixbuf_small_video=Pixbuf.new_from_file_at_size(path,self.series_image_small_size,self.series_image_small_size)
			self.image_big=Gtk.Image.new_from_pixbuf(Pixbuf.new_from_file_at_size(path,self.series_image_big_size+60,self.series_image_big_size))
			shutil.copy2(path,self.path+"/.folder")
			
	def get_image(self):
		return self.pixbuf_small_video
	
	def get_big_image(self):
		return self.image_big
	
	def get_name(self):
		return self.name

	def get_videos(self):
		return self.videos
		
	def get_o_played_stats(self):
		"""returns played,total,percent """
		
		self.update_not_hiden_videos()
		
		
		i=0.00
		for video in self.videos:
			if video.get_o_played() and video.get_display():
				i+=1.00
		
		if i > 0 and self.nb_videos > 0:
			return int(i),self.nb_videos,(i/self.nb_videos)
		else:
			return 0,self.nb_videos,0

	def get_r_played_stats(self):
		"""returns played,total,percent """
		
		self.update_not_hiden_videos()
		
		i=0.00
		for video in self.videos:
			if video.get_r_played() and video.get_display():
				i+=1.00
		
		if i > 0 and self.nb_videos > 0:
			return int(i),self.nb_videos, (i/self.nb_videos)
		else:
			return 0,self.nb_videos,0

	def get_o_episode(self):
		for video in self.videos:
			if not video.get_o_played() and video.get_play() and video.get_path() and video.get_display():
				return video
				
	def get_r_episode(self):

		# check if there are videos to play
		there_are_videos=False
		for video in self.videos:
			if not video.get_r_played() and video.get_play() and video.get_path() and video.get_display():
				there_are_videos=True
				
		# return some video
		if there_are_videos:
			while True:
				random_ep=random.randint(0, self.nb_videos-1)
				random_video=self.videos[random_ep]
				
				if not random_video.get_r_played() and random_video.get_play() and random_video.get_path():
					return random_video
		else:
			return False
					
					
	def get_nb_videos(self):
		return self.nb_videos
	

	def write_data(self):		
		if not os.path.exists(FOLDER_LIST_PATH):
			 os.mkdir(FOLDER_LIST_PATH) 
		
		with open(SERIE_PATH.format(self.name), 'w') as f:
			csv_list=csv.writer(f,delimiter='|')
			csv_list.writerow([self.path,self.recursive,self.random,self.keep_playing])
			for video in self.videos:
				csv_list.writerow([	video.get_id(),
									video.get_path(),
									video.get_play(),
									video.get_o_played(),
									video.get_r_played(),
									video.get_position(),
									video.get_display(),
									])
		
		
	def get_path(self):
		return self.path


class Video(object):
	
	def __init__(self,path,id):
		
		self.id=id
		
		try:
			self.name=os.path.basename(path)
			self.dir_path=os.path.dirname(path)
		except Exception,e:
			print "wrong path"
			self.name="None"
			self.dir_path="None"
	
		self.path=path	
			
			
		self.play=True
		self.o_played=False
		self.r_played=False
		self.position=0
		self.display=True

		self.update_state()

	
	def load_info(self,play,o_played,r_played,position,display):
	
		if play=="false" or play=="False":
			self.set_play(False)
			
		if o_played=="true" or o_played=="True":
			self.set_o_played(True)
			
		if r_played=="true" or r_played=="True":
			self.set_r_played(True)
			
		if position > 0:
			self.set_position(position)
			
		if display != "True" and display != "true":
			self.set_display(False)
	
	
	def update_state(self):

		if os.path.exists(self.path):
			self.state=Gtk.STOCK_APPLY
		else:
			self.state=Gtk.STOCK_DIALOG_WARNING
		
		
	def set_path(self,path):
		self.path=path
		self.name=os.path.basename(path)
		self.update_state()
		
	def set_state_new(self):
		self.state=Gtk.STOCK_ADD
		
	def set_state(self, pixbuf):
		self.state=pixbuf
	
	def get_position(self):
		return self.position
		
	def set_position(self,pos):
		if pos < 1 and pos >= 0:
			self.position=pos
		else:
			print self.name,"wrong set_position"
			
	def get_display(self):
		return self.display
		
	def set_display(self,bool):
		self.display=bool
	
	def get_path(self):
		return self.path
			
	def get_name(self):
		return self.name
		
	def get_state(self):
		return self.state
		
	def get_play(self):
		return self.play
		
	def get_o_played(self):
		return self.o_played

	def get_r_played(self):
		return self.r_played
		
	def set_r_played(self,bool):
		self.r_played=bool
		
	def set_o_played(self,bool):
		self.o_played=bool
		
	def set_play(self,bool):
		self.play=bool
		
	def set_id(self,integer):
		if int(integer) < 0:
			print "video id error "+self.name
		else:
			self.id=int(integer)
		
	def get_id(self):
		return self.id
